<?php
/**
 * MultiQA
 * @copyright 2013-4 Arantor
 * @license 3-clause BSD
 *
 * @version 1.0
 */

/*
 *	This script prepares the database for all the tables and other database changes are required.
 *
 *	NOTE: This script is meant to run using the database elements of the package-info.xml file. This is so
 *	that admins have the choice to uninstall any database data installed with the mod. Also, since using the database
 *	elements automatically calls on db_extend('packages'), we will only be calling that if we are running this script standalone.
 *
 *  Parts of this are shamelessly borrowed from SimpleDesk, other parts from my own contributions to SMF 2.1.
 */

/**
 *	Before attempting to execute, this file attempts to load SSI.php to enable access to the database functions.
*/

// If we have found SSI.php and we are outside of SMF, then we are running standalone.
if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
	require_once(dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF')) // If we are outside SMF and can't find SSI.php, then throw an error
	die('<b>Error:</b> Cannot install - please verify you put this file in the same place as SMF\'s SSI.php.');

if (SMF == 'SSI')
	db_extend('packages');

$tables = array();
$tables[] = array(
	'table_name' => '{db_prefix}qanda',
	'columns' => array(
		db_field('id_question', 'smallint', 0, true, true),
		db_field('lngfile', 'varchar', 255),
		db_field('question', 'varchar', 255),
		db_field('answers', 'text'),
	),
	'indexes' => array(
		array(
			'columns' => array('id_question'),
			'type' => 'primary',
		),
		array(
			'columns' => array('lngfile'),
			'type' => 'index',
		),
	),
	'if_exists' => 'ignore',
	'error' => 'fatal',
	'parameters' => array(),
);

// Create new tables, if any
foreach ($tables as $table)
{
	$smcFunc['db_create_table']($table['table_name'], $table['columns'], $table['indexes'], $table['parameters'], $table['if_exists'], $table['error']);

	// Because of issues with SMF at least in 2.0 RC5, users coming from older installs may not have all columns as if_exists => update doesn't appear to work.
	// So, for every column, add it to the columns addition - and let SMF deal with it that way.
	foreach ($table['columns'] as $table_info)
		$columns[] = array(
			'table_name' => $table['table_name'],
			'column_info' => $table_info,
			'parameters' => array(),
			'if_exists' => 'ignore',
			'error' => 'fatal',
		);
}

// Now port any existing questions into this new-fangled set up. Assume the forum's default language.
global $language;
$insert_lang = strtr($language, array('-utf8' => ''));
$insert_rows = array();

// First, see if we already have questions, if so, do nothing.
$request = $smcFunc['db_query']('', '
	SELECT COUNT(id_question)
	FROM {db_prefix}qanda');
list($count) = $smcFunc['db_fetch_row']($request);
if ($count == 0)
{
	$request = $smcFunc['db_query']('', '
		SELECT id_comment, body AS question, recipient_name AS answer
		FROM {db_prefix}log_comments
		WHERE comment_type = {string:ver_test}',
		array(
			'ver_test' => 'ver_test',
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$insert_rows[] = array(
			$insert_lang,
			$row['question'],
			serialize(array($row['answer'])),
		);
	}
	$smcFunc['db_free_result']($request);

	if (!empty($insert_rows))
	{
		$smcFunc['db_insert']('',
			'{db_prefix}qanda',
			array(
				'lngfile' => 'string', 'question' => 'string', 'answers' => 'string',
			),
			$insert_rows,
			array('id_question')
		);
	}
}

// Are we done?
if (SMF == 'SSI')
	echo 'Database changes are complete!';

function db_field($name, $type, $size = 0, $unsigned = true, $auto = false)
{
	$fields = array(
		'varchar' => array(
			'auto' => false,
			'type' => 'varchar',
			'size' => $size == 0 ? 50 : $size,
			'null' => false,
		),
		'text' => array(
			'auto' => false,
			'type' => 'text',
			'null' => false,
		),
		'mediumtext' => array(
			'auto' => false,
			'type' => 'mediumtext',
			'null' => false,
		),
		'tinyint' => array(
			'auto' => $auto,
			'type' => 'tinyint',
			'default' => 0,
			'size' => empty($unsigned) ? 4 : 3,
			'unsigned' => $unsigned,
			'null' => false,
		),
		'smallint' => array(
			'auto' => $auto,
			'type' => 'smallint',
			'default' => 0,
			'size' => empty($unsigned) ? 6 : 5,
			'unsigned' => $unsigned,
			'null' => false,
		),
		'mediumint' => array(
			'auto' => $auto,
			'type' => 'mediumint',
			'default' => 0,
			'size' => 8,
			'unsigned' => $unsigned,
			'null' => false,
		),
		'int' => array(
			'auto' => $auto,
			'type' => 'int',
			'default' => 0,
			'size' => empty($unsigned) ? 11 : 10,
			'unsigned' => $unsigned,
			'null' => false,
		),
	);

	$field = $fields[$type];
	$field['name'] = $name;

	return $field;
}

?>